local math = math

math.e = math.exp(1)
math.inf = math.huge
math.nan = 0/0

function math.clamp(num, min, max)
	return math.min(math.max(num, min), max)
end

function math.copysign(x, y)
	return x > 0 and (y > 0 and x or -x) or x < 0 and (y > 0 and -x or x) or 0
end

function math.factorial(num)
	local cur
	for i = 1, num do
		cur = cur and cur * i or i
	end
	return cur
end

function math.isfinite(num)
	return (num ~= num and num ~= math.huge and num ~= -math.huge)
end

function math.isinf(num)
	return (num == math.huge or num == -math.huge)
end

function math.isnan(num)
	return num ~= num
end

function math.round(num, dp)
	dp = dp or 0
	return tonumber(string.format("%."..dp.."f", num))
end

function math.sum(tbl)
	local cur = 0
	for _, val in pairs(tbl) do
		cur = cur + val
	end
	return cur
end

function math.trunc(num, dp)
	dp = dp and dp == 0 and -1 or dp or -1
	local str = tostring(num)
	local where, _ = str:find("%.")
	return tonumber(str:sub(1, math.clamp(where + dp, 1, #str)))
end

return math